/// Things used in J2D and J3D that should not be exposed to the user of the library.
#pragma once

#include "glad/glad.h"
#include <array>
#include <vector>
#include "J3ML/LinearAlgebra/Vector2.hpp"
#include "JGL/types/RenderTarget.h"
#include "JGL/types/Light.h"
#include "JGL/logger/logger.h"

namespace JGL {
    inline constexpr std::array<const LightBase*, 8> empty_light_array = { nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr };
    inline bool inJ2D = false;
    inline bool inJ3D = false;
    std::vector<Vector3> TriangleMeshVertexNormals(const Vector3* vertices, const size_t& vertex_count, const unsigned int* indices, const size_t& index_count);
}

namespace JGL::OpenGLState {
    inline JGL::RenderTarget* render_target = nullptr;
    inline Vector2 window_size;

    inline GLfloat oldColor[4] = {0, 0, 0, 1};
    inline GLfloat baseColor[4] = {1, 1, 1, 1};

    inline GLuint current_fbo = 0;
    inline GLint viewport[4] = {0, 0, 0, 0};

    inline bool wasTexture2DEnabled = false;
    inline bool wasTextureCoordArrayEnabled = false;
    inline bool wasNormalArrayEnabled = false;

    inline bool wasDepthTestEnabled = false;
    inline bool wasVertexArraysEnabled = false;
    inline bool wasCullFaceEnabled = false;
    inline bool wasBlendEnabled = false;
    inline bool wasColorArrayEnabled = false;
    inline GLint activeTextureUnit = 0;
}

namespace JGL::J2D {
    inline std::array<const LightBase*, 8> required_lights;
    inline std::vector<const LightBase*> light_array;
}

namespace JGL::J3D {
    // List of lights required for each objects in the scene. up-to 8. For example, the sun. Or a flash-light.
    inline std::array<const LightBase*, 8> required_lights;
    // List of all lights in the scene.
    inline std::vector<const LightBase*> optional_lights;
    inline float far_plane = 0;
    inline float fov = 0;
    // Enables lighting and selects the correct lights to use.
    void SelectLights(const Vector3& position);
    // Resets the GL lights to default and disables them. Then, disables lighting.
    void ResetLights();
    [[nodiscard]] bool UsingLighting();

}